/* $Id: glu.c,v 1.21.2.3 2001/05/14 19:11:33 brianp Exp $ */

/*
 * Mesa 3-D graphics library
 * Version:  3.4
 * Copyright (C) 1995-2000  Brian Paul
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

private import opengl;
private import math;

void gluPerspective(GLdouble fovy, GLdouble aspect,
					GLdouble zNear, GLdouble zFar)
{
	GLdouble xmin, xmax, ymin, ymax;

	ymax = zNear * math.tan(fovy * math.PI / 360.0);
	ymin = -ymax;

	xmin = ymin * aspect;
	xmax = ymax * aspect;

	glFrustum(xmin, xmax, ymin, ymax, zNear, zFar);
}

void gluLookAt(GLdouble eyex, GLdouble eyey, GLdouble eyez,
			   GLdouble centerx, GLdouble centery, GLdouble centerz,
			   GLdouble upx, GLdouble upy, GLdouble upz)
{
	GLdouble[16] m;
	GLdouble[3] x, y, z;
	GLdouble mag;

	/* Make rotation matrix */

	/* Z vector */
	z[0] = eyex - centerx;
	z[1] = eyey - centery;
	z[2] = eyez - centerz;
	mag = sqrt(z[0] * z[0] + z[1] * z[1] + z[2] * z[2]);
	if (mag) {                   /* mpichler, 19950515 */
		z[0] /= mag;
		z[1] /= mag;
		z[2] /= mag;
	}

	/* Y vector */
	y[0] = upx;
	y[1] = upy;
	y[2] = upz;

	/* X vector = Y cross Z */
	x[0] = y[1] * z[2] - y[2] * z[1];
	x[1] = -y[0] * z[2] + y[2] * z[0];
	x[2] = y[0] * z[1] - y[1] * z[0];

	/* Recompute Y = Z cross X */
	y[0] = z[1] * x[2] - z[2] * x[1];
	y[1] = -z[0] * x[2] + z[2] * x[0];
	y[2] = z[0] * x[1] - z[1] * x[0];

	/* mpichler, 19950515 */
	/* cross product gives area of parallelogram, which is < 1.0 for
	 * non-perpendicular unit-length vectors; so normalize x, y here
	 */

	mag = sqrt(x[0] * x[0] + x[1] * x[1] + x[2] * x[2]);
	if (mag) {
		x[0] /= mag;
		x[1] /= mag;
		x[2] /= mag;
	}

	mag = sqrt(y[0] * y[0] + y[1] * y[1] + y[2] * y[2]);
	if (mag) {
		y[0] /= mag;
		y[1] /= mag;
		y[2] /= mag;
	}

	m[0*4+0] = x[0];
	m[1*4+0] = x[1];
	m[2*4+0] = x[2];
	m[3*4+0] = 0.0;
	m[0*4+1] = y[0];
	m[1*4+1] = y[1];
	m[2*4+1] = y[2];
	m[3*4+1] = 0.0;
	m[0*4+2] = z[0];
	m[1*4+2] = z[1];
	m[2*4+2] = z[2];
	m[3*4+2] = 0.0;
	m[0*4+3] = 0.0;
	m[1*4+3] = 0.0;
	m[2*4+3] = 0.0;
	m[3*4+3] = 1.0;

	glMultMatrixd(m);

	/* Translate Eye to Origin */
	glTranslated(-eyex, -eyey, -eyez);

}
